using System;
using System.Collections.Generic;
using System.Text.Json;
using System.Text.RegularExpressions;
using Microsoft.AspNetCore.Http;

using SlackAPI;
using SlackAPI.RPCMessages;

using sera_slackbot.Library;

namespace sera_slackbot.Commands
{
    public class PointingPokerBotCommand : IBotCommand, IBotInteraction
    {
        private JsonFileReader reader;
        private JsonFileSaver saver;

        private string channelId;
        private string userId;
        private string text;
        private string message;

        private string messageTs;

        private string task;
        public PointingPokerBotCommand(JsonFileReader reader, JsonFileSaver saver)
        {
            this.reader = reader;
            this.saver = saver;
        }

        public void ExtractValidData(JsonElement slackEvent)
        {
            this.text = slackEvent.GetProperty("text").ToString();
            this.channelId = slackEvent.GetProperty("channel").ToString();
            this.userId = slackEvent.GetProperty("user").ToString();
            this.messageTs = slackEvent.GetProperty("ts").ToString();
        }

        public bool WillTrigger(JsonElement slackEvent)
        {
            var text = slackEvent.GetProperty("text").ToString();
            return text.Contains("point", StringComparison.InvariantCultureIgnoreCase) || text.Contains("estimat", StringComparison.InvariantCultureIgnoreCase);
        }

        public dynamic Run(SlackClient client)
        {
            task = new Regex("\".*?\"").Matches(text)[0].ToString().Trim('"');

            this.saver.Save("pointingpokersession.json", new
            {
                creator_id = userId,
                channel_id = channelId,
                estimate_ts = messageTs,
                user_story = task,
                user_rating = new List<dynamic> { }
            });


            PostMessage(client, BuildEstimationView(task));
            PostEphemerialMessage(client, BuildManagementView(task));

            return new
            {
                code = StatusCodes.Status200OK,
                body = ""
            };
        }

        void PostMessage(SlackClient client, string blocksJson)
        {
            var request = new List<Tuple<string, string>>();
            request.Add(new Tuple<string, string>("channel", channelId));
            request.Add(new Tuple<string, string>("user", userId));
            request.Add(new Tuple<string, string>("blocks", blocksJson));

            var callback = new Action<PostMessageResponse>((response) =>
            {
            });
            client.APIRequestWithToken(callback, request.ToArray());
        }

        void PostEphemerialMessage(SlackClient client, string blocksJson)
        {
            var request = new List<Tuple<string, string>>();
            request.Add(new Tuple<string, string>("channel", channelId));
            request.Add(new Tuple<string, string>("user", userId));
            request.Add(new Tuple<string, string>("blocks", blocksJson));

            var callback = new Action<PostEphemeralResponse>((response) =>
            {
            });
            client.APIRequestWithToken(callback, request.ToArray());
        }

        private string BuildEstimationView(string taskToEstimate)
        {
            var blocks = new List<dynamic>();
            blocks.Add(new
            {
                type = "section",
                text = new
                {
                    type = "mrkdwn",
                    text = "Hello developers! It's time to *estimate*!"
                }
            });
            blocks.Add(new
            {
                type = "section",
                text = new
                {
                    type = "mrkdwn",
                    text = $"The user story is \"{taskToEstimate}\"."
                }
            });

            var actionElements = new List<dynamic>();
            actionElements.Add(new
            {
                type = "button",
                text = new
                {
                    type = "plain_text",
                    text = "Extra Small (< 1 week)",
                    emoji = false
                },
                value = "pointingpoker:1"
            });

            actionElements.Add(new
            {
                type = "button",
                text = new
                {
                    type = "plain_text",
                    text = "Small (1 - 2 weeks)",
                    emoji = false
                },
                value = "pointingpoker:2"
            });
            actionElements.Add(new
            {
                type = "button",
                text = new
                {
                    type = "plain_text",
                    text = "Medium (2 - 4 weeks)",
                    emoji = false
                },
                value = "pointingpoker:3"
            });
            actionElements.Add(new
            {
                type = "button",
                text = new
                {
                    type = "plain_text",
                    text = "Large (4 - 8 weeks)",
                    emoji = false
                },
                value = "pointingpoker:4"
            });
            actionElements.Add(new
            {
                type = "button",
                text = new
                {
                    type = "plain_text",
                    text = "Extra Large (8+ weeks)",
                    emoji = false
                },
                value = "pointingpoker:5"
            });

            blocks.Add(new
            {
                type = "actions",
                elements = actionElements.ToArray()
            });

            var jsonText = JsonSerializer.Serialize(blocks);
            return jsonText;
        }

        private string BuildManagementView(string taskToEstimate)
        {
            var blocks = new List<dynamic>();
            blocks.Add(new
            {
                type = "section",
                text = new
                {
                    type = "mrkdwn",
                    text = "Psst! When everyone has finished, let me know and I'll yell out the average.\n" +
                    $"This is for the user story: \"{taskToEstimate}\"."
                }
            });

            var actionElements = new List<dynamic>();
            actionElements.Add(new
            {
                type = "button",
                text = new
                {
                    type = "plain_text",
                    text = "Get the Average",
                    emoji = false
                },
                value = "pointingpoker:finish",
                style = "primary",
            });

            actionElements.Add(new
            {
                type = "button",
                text = new
                {
                    type = "plain_text",
                    text = "Delete the estimate",
                    emoji = false,
                },
                value = "pointingpoker:delete",
                style = "danger"
            });

            blocks.Add(new
            {
                type = "actions",
                elements = actionElements.ToArray()
            });

            var jsonText = JsonSerializer.Serialize(blocks);
            return jsonText;
        }

        public dynamic Interaction(SlackClient client, JsonElement payload, string actionValue)
        {
            var session = this.reader.Read("pointingpokersession.json");

            // var triggerId = payload.GetProperty("trigger_id").ToString();
            // var containerTs = container.GetProperty("message_ts").ToString();

            dynamic response = new { code = StatusCodes.Status200OK };
            switch (actionValue)
            {
                case "1":
                case "2":
                case "3":
                case "4":
                case "5":
                    response = PointingPokerUpdate(client, payload, session, actionValue);
                    break;
                case "finish":
                    response = PointingPokerFinish(client, payload, session);
                    break;
                case "delete":
                    response = PointingPokerDelete(client, payload, session);
                    break;
            }
            return response;
        }

        private dynamic PointingPokerUpdate(SlackClient client, dynamic payload, dynamic session, string value)
        {
            var userId = payload.GetProperty("user").GetProperty("id").ToString();

            JsonElement userRatings = session.GetProperty("user_rating");
            var newRatings = new List<dynamic>();
            foreach (var user in userRatings.EnumerateArray())
            {
                var curUserId = user.GetProperty("id").ToString();
                var curValue = user.GetProperty("value").ToString();
                if (curUserId != userId)
                {
                    newRatings.Add(new
                    {
                        id = curUserId,
                        value = curValue
                    });
                }
            }
            newRatings.Add(new
            {
                id = userId,
                value = value
            });

            this.saver.Save("pointingpokersession.json", new
            {
                creator_id = session.GetProperty("creator_id").ToString(),
                channel_id = session.GetProperty("channel_id").ToString(),
                estimate_ts = session.GetProperty("estimate_ts").ToString(),
                user_story = session.GetProperty("user_story").ToString(),
                user_rating = newRatings.ToArray()
            });

            dynamic response = new
            {
                code = StatusCodes.Status200OK
            };

            return response;
        }

        private dynamic PointingPokerFinish(SlackClient client, dynamic payload, dynamic session)
        {
            JsonElement userRatings = session.GetProperty("user_rating");
            var total = 0;
            var count = 0;
            foreach (var user in userRatings.EnumerateArray())
            {
                count++;
                total += int.Parse(user.GetProperty("value").ToString());
            }

            JsonElement container = payload.GetProperty("container");
            // var estimateTs = session.GetProperty("estimate_ts");
            // var userId = payload.GetProperty("user").GetProperty("id").ToString();
            var channelId = container.GetProperty("channel_id").ToString();

            // var parameters = new List<Tuple<string, string>>();
            // parameters.Add(new Tuple<string, string>("ts", estimateTs));
            // parameters.Add(new Tuple<string, string>("channel", channelId));
            // parameters.Add(new Tuple<string, string>("as_user", "true"));

            // client.APIRequestWithToken<DeletedResponse>(response =>
            // {
            //     Console.WriteLine("Should have deleted " + estimateTs + " " + response.error);
            // }, parameters.ToArray());

            // var clearMessage = new
            // {
            //     response_type = "ephemeral",
            //     text = "",
            //     replace_original = true,
            //     delete_original = true,
            // };

            // {
            //     code = StatusCodes.Status200OK,
            //     body = JsonSerializer.Serialize(clearMessage)
            // };


            dynamic response = new
            {
                code = StatusCodes.Status200OK,
                body = ""
            };

            var roundedAverage = Math.Round((decimal)total / count);
            var userStory = session.GetProperty("user_story").ToString();

            client.PostMessage(response =>
            {
                client.PostMessage(response =>
                {

                }, channelId, $"The user story '{userStory}' got an average of {roundedAverage}.");
            }, channelId, $"Finished!");
            return response;
        }

        private dynamic PointingPokerDelete(SlackClient client, dynamic payload, dynamic session)
        {

            JsonElement container = payload.GetProperty("container");
            var estimateTs = session.GetProperty("estimate_ts");
            var userId = payload.GetProperty("user").GetProperty("id").ToString();
            var channelId = container.GetProperty("channel_id").ToString();

            var parameters = new List<Tuple<string, string>>();
            parameters.Add(new Tuple<string, string>("ts", estimateTs));
            parameters.Add(new Tuple<string, string>("channel", channelId));
            parameters.Add(new Tuple<string, string>("as_user", "true"));

            client.APIRequestWithToken<DeletedResponse>(response =>
            {
                Console.WriteLine("Should have deleted " + estimateTs + " " + response.error);
            }, parameters.ToArray());

            var clearMessage = new
            {
                response_type = "ephemeral",
                text = "",
                replace_original = true,
                delete_original = true,
            };

            dynamic response = new
            {
                code = StatusCodes.Status200OK,
                body = JsonSerializer.Serialize(clearMessage)
            };

            return response;
        }
    }
}
