using System;
using Microsoft.AspNetCore.Http;
using System.Web;

using SlackAPI;
using System.Text.Json;
using System.Collections.Generic;

using sera_slackbot.Library;

namespace sera_slackbot.Commands
{
    public class ScoreKeeperBotCommand : IBotCommand
    {
        private JsonFileReader reader;
        private JsonFileSaver saver;
        private string channelId;
        private string userId;
        private string text;
        private string message;
        public ScoreKeeperBotCommand(JsonFileReader reader, JsonFileSaver saver)
        {
            this.reader = reader;
            this.saver = saver;
        }

        public void ExtractValidData(JsonElement slackEvent)
        {
            this.text = slackEvent.GetProperty("text").ToString();
            this.channelId = slackEvent.GetProperty("channel").ToString();
            this.userId = slackEvent.GetProperty("user").ToString();
        }

        public bool WillTrigger(JsonElement slackEvent)
        {
            var text = slackEvent.GetProperty("text").ToString();
            return text.Contains("point", StringComparison.InvariantCultureIgnoreCase) && text.Contains("add", StringComparison.InvariantCultureIgnoreCase);
        }

        public dynamic Run(SlackClient client)
        {
            var words = text.Split(" ");
            var index = Array.FindIndex(words, word => word.Equals("to", StringComparison.InvariantCultureIgnoreCase));
            char[] charsToTrim = { '"', '.', ',', '?', '!', ' ' };
            var displayTarget = words[index + 1].Trim(charsToTrim);
            var target = HttpUtility.UrlEncode(displayTarget.Trim('\u003C').Trim('\u003E').ToLower());

            JsonElement scores;
            try
            {
                scores = this.reader.Read("scorekeeper.json");
            }
            catch (Exception e)
            {
                Console.WriteLine("Something went wrong, creating new dictionary " + e.Message);
                scores = new JsonElement();
            }

            var i = -1;
            var foundIndex = i;

            var list = new List<dynamic>();

            foreach (var score in scores.EnumerateArray())
            {
                i++;
                list.Add(new
                {
                    key = score.GetProperty("key").ToString(),
                    value = score.GetProperty("value").GetInt32()
                });
                if (score.GetProperty("key").ToString().Equals(target))
                {
                    foundIndex = i;
                }
            };

            var newScore = 0;

            if (foundIndex == -1)
            {
                list.Add(new
                {
                    key = target,
                    value = 1
                });
                newScore = 1;
            }
            else
            {
                list[foundIndex] = new
                {
                    key = list[foundIndex].key,
                    value = list[foundIndex].value + 1
                };
                newScore = list[foundIndex].value + 1;
            }

            this.saver.Save("scorekeeper.json", list);
            if (displayTarget.StartsWith('@'))
            {
                displayTarget = $"\u003C{displayTarget}\u003E";
            }
            message = $"Ok. I've added a point to {displayTarget}. The new score is {newScore}.";

            client.PostMessage(response => { }, channelId, message);
            return new
            {
                code = StatusCodes.Status200OK,
                body = ""
            };
        }
    }
}
